#include <stdio.h>
#include <stdlib.h>
#include <roxml.h>
#include "structures.h"
#include "competence.h"


/*!
* \file competence.c
* \brief Fichier contenant le code des fonctions liees aux arbres de comptences.
*/

/*! Copie de la struct Competence pour travailler

typedef struct Competence{
    char nomComp[TAILLE_NOMS];
    char id[TAILLE_ID];
    unsigned int prix_competence;
    struct Competence *frere;
    struct Competence *enfant;
    struct Effet_Competence *effet;
    boolean connu;
}Competence;

*/

/*!
 * \fn int init_Competence(Competence *noeud)
 * \brief La fonction initialise une competence.
 *
 * \return Un entier.
 */
Competence* init_Competence()
{
    Competence* noeud = malloc(sizeof(Competence));

    noeud->prix_competence = 0;
    noeud->frere = NULL;
    noeud->enfant = NULL;
    noeud->effet = malloc(sizeof(Effet_Competence));
    noeud->connu = 0;

    return noeud;
}

/* -------------------------------------------------------*/
/* Fonctions de test des competences */
/* -------------------------------------------------------*/

/*!
 * \fn int estVide(Competence *c)
 * \brief La fonction indique si une competence n'a pas d'enfants.
 *
 * \return vrai ou faux.
 */
int estVide(Competence *c)
{
    return(c->enfant==NULL);
}

/*!
 * \fn int aUnFrere(Competence *c)
 * \brief La fonction indique si une competence a un frre.
 *
 * \return vrai ou faux.
 */
int aUnFrere(Competence *c)
{
    return(c->frere!=NULL);
}

/*!
 * \fn int set_comp_enfant(Competence *parent, Competence *nouvel_enfant)
 * \brief La fonction attribue un enfant  un noeud d'un arbre de competences.
 *
 * \param Une competence parent, une competence enfant.
 * \return Un entier.
 */
 int set_comp_enfant(Competence *parent, Competence *nouvel_enfant)
{
    if(estVide(parent))
    {
        parent->enfant = nouvel_enfant;
    }
    else
    {
        set_comp_frere(parent->enfant, nouvel_enfant);
    }
    return 0;
}

/*!
 * \fn set_comp_frere(Competence *noeud, Competence *nouveau_frere)
 * \brief La fonction attribue un frere  un noeud d'un arbre de competences.
 *
 * \param Une competence parent, une competence frere.
 * \return Un entier.
 */
int set_comp_frere(Competence *noeud, Competence *nouveau_frere)
{
    if(aUnFrere(noeud))
    {
        set_comp_frere(noeud->frere, nouveau_frere);
    }
    else
    {
        noeud->frere = nouveau_frere;
    }
    return 0;
}

/*!
 * \fn int set_comp_name(Competence *comp, char *nom_donne)
 * \brief La fonction definit le nom d'une competence.
 *
 * \param Une competence, une chaine de caracteres.
 * \return Un entier.
 */
int set_comp_name(Competence *comp, char *nom_donne)
{
    strcpy(comp->nomComp, nom_donne);
    return 0;
}

/*!
 * \fn int set_comp_id(Competence *comp, char *nom_donne)
 * \brief La fonction definit l'ID d'une competence.
 *
 * \param Une competence, une chaine de caracteres.
 * \return Un entier.
 */
int set_comp_id(Competence *comp, char *nom_donne)
{
    strcpy(comp->id, nom_donne);
    return 0;
}

/*!
 * \fn int set_comp_prix(Competence *comp, int prix)
 * \brief La fonction definit le prix d'une competence.
 *
 * \param Une competence, un entier.
 * \return Un entier.
 */
int set_comp_prix(Competence *comp, int prix)
{
    comp->prix_competence=prix;
    return 0;
}

/*!
 * \fn int set_comp_effet(Competence *comp, Effet_Competence *nouvel_effet)
 * \brief La fonction ajoute un effet a une competence.
 *
 * \param Une competence, un effet.
 * \return Un entier.
 */
int set_comp_effet(Competence *comp, Effet_Competence *nouvel_effet)
{
    comp->effet = nouvel_effet;
    return 0;
}


/*!
 * \fn int taille(Competence *noeud)
 * \brief La fonction renvoie le nombre d'enfants totaux d'une competence.
 *
 * \param Une competence.
 * \return Un entier positif.
 */
int taille(Competence *noeud)
{
    Competence *enfant;
    int compteur = 1;
    for (enfant = noeud->enfant; enfant != NULL; enfant = enfant->frere)
        compteur += taille(enfant);
    return compteur;
}

/*!
 * \fn unsigned int hauteur(Competence *noeud)
 * \brief La fonction indique la hauteur d'une competence dans l'arbre.
 *
 * \param Une competence.
 * \return Un entier positif.
 */
unsigned int hauteur(Competence *noeud)
{
    if (!estVide(noeud))
    {
        int compteur = 1;
        Competence *enfant;
        for (enfant = noeud->enfant; enfant != NULL; enfant = enfant->frere)
        {
            if (hauteur(enfant) > compteur)
                compteur = 1 + hauteur(enfant);
        }
        return compteur;
    }
    else
    {
        return 0;
    }
}

/*!
 * \fn boolean est_connue(Competence *c)
 * \brief La fonction indique si une competence est connue ou non.
 *
 * \param Une competence.
 * \return vrai ou faux.
 */
boolean est_connue(Competence *c)
{
    return (c->connu);
}


/*!
 * \fn unsigned int * NbEnfants(Competence *noeud)
 * \brief La fonction renvoie le nombre d'enfants d'une competence.
 * \param Une competence.
 * \return Un entier positif.
 */
unsigned int NbEnfants(Competence *noeud)
{
    unsigned int nb;
    if (estVide(noeud))
    {
        nb = 0;
    }
    else
    {
        Competence *enfant;
        for(enfant=noeud->enfant; enfant!= NULL; enfant=enfant->frere)
        nb++;
    }
    return nb;
}

/*!
 * \fn boolean * est_complet(Competence *noeud)
 * \brief La fonction verifie si l'arbre est complet ou non.
 *
 * \param Une competence.
 * \return Un booleen.
 */
boolean est_complet(Competence *noeud)
{
    Competence *enfant;
    for(enfant=noeud->enfant; enfant!= NULL; enfant=enfant->frere)
    {
        if(!est_complet(enfant))
            return 0;
    }
    return 1;
}

/*!
 * \fn boolean * est_complet(Competence *noeud)
 * \brief La fonction renvoie la liste des comptences non connues se trouvant sur le chemin d'une comptence slectionne.
 *
 * \param Une competence.
 * \return Un tableau de competences.
 */
int chemin_auto_competence(Competence* c)         /* A FAIRE */
{
    return 0;
}

/*!
 * \fn Effet_Competence * get_effet(Competence *c)
 * \brief Renvoie l'effet d'une competence
 *
 * \param Une competence.
 * \return Un Effet_Competence.
 */
Effet_Competence * get_effet(Competence *c)
{
    return c->effet;
}

/*!
 * \fn int Extraire_Comp_Tree(node_t *arbreXML, Competence* arbreComp)
 * \brief Lie tous les noeuds d'un arbre de competences.
 *
 * \param Un pointeur sur un arbre, un pointeur sur une competence.
 * \return Un entier.
 */
int Extraire_Comp_Tree(node_t *arbreXML, Competence* arbreComp)
{
    int nb_Comp=0, i=0;

    nb_Comp = roxml_get_chld_nb(arbreXML);

    for(i=0; i<nb_Comp; i++)
    {
        node_t *brancheXML;
        Competence *comp = init_Competence();

        brancheXML = roxml_get_chld(arbreXML, NULL, i);
        //printf("--> Extraction de la competence %d sur %d \n", i+1, nb_Comp);

        set_comp_name(comp, roxml_get_content(roxml_get_attr(brancheXML,"nom",0),NULL,0,NULL));
        set_comp_id(comp, roxml_get_content(roxml_get_attr(brancheXML,"id",0),NULL,0,NULL));
        set_comp_prix(comp, atoi(roxml_get_content(roxml_get_attr(brancheXML,"prix",0),NULL,0,NULL)));

        //printf("La competence actuelle a pour nom : %s\n", comp->nomComp);
        //printf("La competence actuelle a pour id : %s\n", comp->id);
        //printf("La competence actuelle a pour prix : %d\n", comp->prix_competence);

        Extraire_Comp_Tree(brancheXML, comp);

        set_comp_enfant(arbreComp, comp);
    }
    return 0;
}

/*!
 * \fn Competence* Extraire_Arbres(char *filename)
 * \brief Extrait les arbres de competences depuis un fichier xml.
 *
 * \param Un nom de fichier.
 * \return Un tableau de competences.
 */
Competence* Extraire_Arbres(char *filename)
{
    node_t *rootXML, *Arbres;
    Competence *Arbre_Competences = init_Competence();

    rootXML = roxml_load_doc(filename);
    Arbres = roxml_get_chld(rootXML,"Arbres",0);

    Extraire_Comp_Tree(Arbres, Arbre_Competences);

    return Arbre_Competences;
}


/* Fonction de test (Romain) */

/*
int Test_Competences()
{
        int ret = 0;
        printf("Debut Test Competences\n");

        Competence *Comp_Test_Parent, *Comp_Test_Enfant, *Comp_Test_Petit_Enfant, *Comp_Test_Frere_Enfant;

        Comp_Test_Parent = init_Competence();
        Comp_Test_Enfant = init_Competence();
        Comp_Test_Petit_Enfant = init_Competence();
        Comp_Test_Frere_Enfant = init_Competence();

        if(estVide(Comp_Test_Parent))
        {
            printf("Parent n'a pas d'enfants\n");
        }

        printf("Le nom de parent est : %s\n", Comp_Test_Parent->nomComp);

        set_comp_prix(Comp_Test_Parent, 4);
        set_comp_name(Comp_Test_Parent, "Pepe");
        set_comp_id(Comp_Test_Parent, "ID_PE");

        printf("Le nom de parent est : %s\n", Comp_Test_Parent->nomComp);

        set_comp_name(Comp_Test_Enfant, "Papa");
        set_comp_id(Comp_Test_Enfant, "ID_PA");
        set_comp_name(Comp_Test_Petit_Enfant, "Bebe");
        set_comp_id(Comp_Test_Enfant, "ID_BE");
        set_comp_name(Comp_Test_Frere_Enfant, "Tonton");
        set_comp_id(Comp_Test_Frere_Enfant, "ID_TON");

        printf("Le prix de Parent est %d\n", Comp_Test_Parent->prix_competence);
        printf("Le nom de Parent est %s\n", Comp_Test_Parent->nomComp);
        printf("L'ID de Parent est %s\n", Comp_Test_Parent->id);

        set_comp_enfant(Comp_Test_Parent, Comp_Test_Enfant);

        set_comp_enfant(Comp_Test_Parent, Comp_Test_Frere_Enfant);

        printf("\nL'enfant de %s est %s\n", Comp_Test_Parent->nomComp, Comp_Test_Parent->enfant->nomComp);
        printf("L'autre enfant de %s est %s\n", Comp_Test_Parent->nomComp, Comp_Test_Parent->enfant->frere->nomComp);


        return 0;
}
*/



